<?php

namespace App\Console\Commands;

use App\Vehicle;
use App\Mail\PhotosManquantes;
use Illuminate\Console\Command;
use Illuminate\Support\Facades\Mail;
use Intervention\Image\Exception\NotReadableException;
use Intervention\Image\Facades\Image;
use League\Csv\Reader;

class MailMissingPictures extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'vehicles:email:send {csv_file?} {output_path?}';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Sends missing pictures email';

    private $csv_file_path;
    private $output_path;
    private $missing_images;
    private $localisations;

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
        $this->missing_images = array(
            'A' => array(),
            'B' => array(),
            'C' => array(),
            'D' => array(),
        );
        $this->localisations = array(
            'A' => 'Entrep&ocirc;t est - 4900 M&eacute;tropolitain est',
            'B' => 'Entrep&ocirc;t ouest - 5800 C&ocirc;te-de-Liesse',
            'C' => 'Entrep&ocirc;t Rive-Sud - 3475',
            'D' => 'M&eacute;tropolitain Est - 5600 (Inventaire camion)',
        );
    }

    /**
     * Execute the console command.
     *
     * @return mixed
     */
    public function handle()
    {
        $this->csv_file_path = $this->argument('csv_file') ?? env('CSV_LOCATION');
        $this->output_path = $this->argument('output_path') ?? env('PHOTO_DEST');
        if (!strrpos($this->csv_file_path, DIRECTORY_SEPARATOR)) {
            $this->error("                                              ");
            $this->error("  You must use the full path to the csv file  ");
            $this->error("                                              ");
        } elseif (!strrpos($this->output_path, DIRECTORY_SEPARATOR)) {
            $this->error("                                              ");
            $this->error("      You must use the full output path       ");
            $this->error("                                              ");
        } else {
            $this->scan_csv();
        }
    }

    private function scan_csv()
    {
        $csv_filename = substr($this->csv_file_path, strrpos($this->csv_file_path, DIRECTORY_SEPARATOR) + 1);
        $this->info('Scanning ' . $csv_filename);

        $output = $this->output;

        $reader = Reader::createFromPath($this->csv_file_path, 'r');

        $nb = count($reader);

        $this->info('Scanning vehicles...');
        $bar = $output->createProgressBar($nb);

        $locations = [
            'A' => [],
            'B' => [],
            'C' => [],
            'D' => [],
        ];

        foreach ($reader as $index => $row) {
            if ($index > 0) {
                $vehicle = new Vehicle($row);
                $locations[$vehicle->location][] = $vehicle;
            }
            $bar->advance();
        }

        $this->info('');
        $this->info('Scanning vehicle images...');
        $bar = $output->createProgressBar($nb);

        foreach ($locations as $location => $vehicles) {
            foreach ($vehicles as $vehicle) {
                $this->check_image($vehicle->stock_nb, $location, $vehicle->brand, $vehicle->model, $vehicle->year);
                $bar->advance();
            }
        }

        Mail::to('louis.l@mtlautoprix.net')->send(
            new PhotosManquantes($this->missing_images, $this->localisations)
        );

        $this->info('');
        $this->info('Completed!');
    }

    private function check_image(string $stock_nb, $location, $brand, $model, $year)
    {
	$location = strtoupper($location);
        try {
            if (!file_exists($this->output_path . $stock_nb . '-1179x478.jpg')) {
                if (!isset($this->missing_images[$location]["{$stock_nb} ({$brand} {$model} {$year})"])) {
                    $this->missing_images[$location]["{$stock_nb} ({$brand} {$model} {$year})"] = [];
                }
                $this->missing_images[$location]["{$stock_nb} ({$brand} {$model} {$year})"][] = 1;
            }
            if (!file_exists($this->output_path . $stock_nb . '-2-1179x478.jpg')) {
                if (!isset($this->missing_images[$location]["{$stock_nb} ({$brand} {$model} {$year})"])) {
                    $this->missing_images[$location]["{$stock_nb} ({$brand} {$model} {$year})"] = [];
                }
                $this->missing_images[$location]["{$stock_nb} ({$brand} {$model} {$year})"][] = 2;
            }
            if (!file_exists($this->output_path . $stock_nb . '-3-1179x478.jpg')) {
                if (!isset($this->missing_images[$location]["{$stock_nb} ({$brand} {$model} {$year})"])) {
                    $this->missing_images[$location]["{$stock_nb} ({$brand} {$model} {$year})"] = [];
                }
                $this->missing_images[$location]["{$stock_nb} ({$brand} {$model} {$year})"][] = 3;
            }
            if (!file_exists($this->output_path . $stock_nb . '-4-1179x478.jpg')) {
                if (!isset($this->missing_images[$location]["{$stock_nb} ({$brand} {$model} {$year})"])) {
                    $this->missing_images[$location]["{$stock_nb} ({$brand} {$model} {$year})"] = [];
                }
                $this->missing_images[$location]["{$stock_nb} ({$brand} {$model} {$year})"][] = 4;
            }
        } catch (NotReadableException $e) { }
    }
}
