<?php

namespace App\Console\Commands;

use App\Vehicle;
use Illuminate\Console\Command;
use Illuminate\Support\Facades\DB;
use Intervention\Image\Exception\NotReadableException;
use Intervention\Image\Facades\Image;
use League\Csv\Reader;
use Mockery\Exception;

class SyncVehicleInventory extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'vehicles:sync {csv_file?} {images_path?} {output_path?}';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Synchronizes MtlAutoPrix website inventory DB with provided CSV file and Images path';

    private $csv_file_path;
    private $images_path;
    private $output_path;

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return mixed
     */
    public function handle()
    {
        $this->csv_file_path = $this->argument('csv_file') ?? env('CSV_LOCATION');
        $this->csv_backup_loc = env('CSV_BACKUP_LOCATION');
        $this->images_path = $this->argument('images_path') ?? env('PHOTO_SRC');
        $this->output_path = $this->argument('output_path') ?? env('PHOTO_DEST');
        if (!strrpos($this->csv_file_path, DIRECTORY_SEPARATOR)) {
            $this->error("                                              ");
            $this->error("  You must use the full path to the csv file  ");
            $this->error("                                              ");
        } elseif (!strrpos($this->images_path, DIRECTORY_SEPARATOR)) {
            $this->error("                                              ");
            $this->error("   You must use the full path to the images   ");
            $this->error("                                              ");
        } elseif (!strrpos($this->output_path, DIRECTORY_SEPARATOR)) {
            $this->error("                                              ");
            $this->error("      You must use the full output path       ");
            $this->error("                                              ");
        } else {
            //$this->import_annual_images();
            $this->import_csv();
        }
    }

    private function import_csv()
    {
        var_dump(DIRECTORY_SEPARATOR);
        $csv_filename = substr($this->csv_file_path, strrpos($this->csv_file_path, DIRECTORY_SEPARATOR) + 1);
        $this->info('Importing ' . $csv_filename);

        $output = $this->output;

        $reader = Reader::createFromPath($this->csv_file_path, 'r');

        $nb = count($reader);

        $this->info('Processing vehicles...');
        $bar = $output->createProgressBar($nb);

        $locations = [
            'A' => [],
            'B' => [],
            'C' => [],
            'D' => [],
        ];

        $stock_nbs = [];

        foreach ($reader as $index => $row) {
            if ($index > 0) {
                $vehicle = new Vehicle($row);
                $stock_nbs[] = $vehicle->stock_nb;
                $vehicle->save();
                $locations[$vehicle->location][] = $vehicle;
            }
            $bar->advance();
        }

        $vehicles_not_in_csv = DB::table('wp_posts')
            ->join('wp_postmeta', 'ID', '=', 'post_id', 'inner')
            ->where('post_type', '=', 'vehicle')
            ->where('meta_key', '=', 'stock_nb')
            ->whereNotIn('meta_value', $stock_nbs)
            ->get();

        $nb_vehicles_not_in_csv = count($vehicles_not_in_csv);

        $this->info('');
        $this->info('Deleting vehicles not in CSV...');
        $bar = $output->createProgressBar($nb_vehicles_not_in_csv);

        foreach ($vehicles_not_in_csv as $vehicle) {
            $vehicle = new Vehicle((array) $vehicle);
            $vehicle->delete();
            $bar->advance();
        }

        $this->info('');
        $this->info('Processing vehicle images...');
        $bar = $output->createProgressBar($nb);

        foreach ($locations as $location => $vehicles) {
            foreach ($vehicles as $vehicle) {
                $this->process_image($vehicle->stock_nb);
            }
        }
   
        $folder_path = "/home/mtlautoprix/lscache";
        function delTree($dir) {
            $files = glob( $dir . '*', GLOB_MARK );
            foreach( $files as $file ){
                if( substr( $file, -1 ) == '/' )
                    delTree( $file );
                else
                    unlink( $file );
            }
            rmdir( $dir );
        }
        delTree($folder_path);
 
        $this->info('');
        $this->info('Completed!');
    }

    private function process_image($stock_nb)
    {
        for ($i = 1; $i <= 11; $i++) {
            try {
                $source = $this->images_path . $stock_nb . '-' . $i . '.JPG';

                $destinationPrefix = $this->output_path . $stock_nb;
                if ($i > 1) {
                    $destinationPrefix = $this->output_path . $stock_nb . '-' . $i;
                }

                $image = Image::make($source);
                $image->orientate()->fit(1179, 478);
                $image->save($destinationPrefix . '-1179x478.jpg');

                $image = Image::make($source);
                $image->orientate()->fit(865, 460);
                $image->save($destinationPrefix . '-865x460.jpg');

                $image = Image::make($source);
                $image->orientate()->fit(865, 640);
                $image->save($destinationPrefix . '-865x640.jpg');

                $image = Image::make($source);
                $image->orientate()->fit(445, 240);
                $image->save($destinationPrefix . '-445x240.jpg');

                $image = Image::make($source);
                $image->orientate()->fit(157, 85);
                $image->save($this->output_path . $stock_nb . '-' . $i . '.jpg');

                if ($i > 1) {
                    $image = Image::make($source);
                    $image->fit(32, 96);
                    $image->save($destinationPrefix . '-32x96.jpg');
                }
            } catch (NotReadableException $e) {}
        }
    }

    private function import_annual_images() {
        ini_set('memory_limit','512M');

        $this->images_path = $this->argument('images_path') ?? env('ANNUEL_SRC');
        $this->output_path = $this->argument('output_path') ?? env('ANNUEL_DEST');

        try {
            if ($handle = opendir($this->images_path)) {

                $output = $this->output;
                $this->info('');
                $this->info('Importing annual images...');

                while (false !== ($entry = readdir($handle))) {
                    if ($entry != "." && $entry != ".." && $this->endsWith($entry, '.JPG')) {
                        $image_main_mobile = Image::make($this->images_path . $entry);
                        $image_main_mobile->orientate()->fit(865, 460);
                        $image_main_mobile->save($this->output_path . $entry);
                    }
                }
                closedir($handle);
            }

        } catch (NotReadableException $e) {}
    }

    private function endsWith($haystack, $needle)
    {
        $length = strlen($needle);
        if ($length == 0) {
            return true;
        }

        return (substr($haystack, -$length) === $needle);
    }
}
